@@ -3,6 +3,22 @@ import sys
 
 import numpy as np
 
+import glymur
+
+# Need to know the openjpeg version.  If openjpeg is not installed, we use
+# '0.0.0'
+OPENJPEG_VERSION = '0.0.0'
+if glymur.lib.openjpeg.OPENJPEG is not None:
+    OPENJPEG_VERSION = glymur.lib.openjpeg.version()
+
+# Need to know of the libopenjp2 version is the official 2.0.0 release and NOT
+# the 2.0+ development version.
+OPENJP2_IS_V2_OFFICIAL = False
+if glymur.lib.openjp2.OPENJP2 is not None:
+    if not hasattr(glymur.lib.openjp2.OPENJP2,
+                   'opj_stream_create_default_file_stream_v3'):
+        OPENJP2_IS_V2_OFFICIAL = True
+
 
 def mse(amat, bmat):
     """Mean Square Error"""
